@@ -30,33 +30,6 @@ bool RemoteLogger::reconnect()
   return true;
 }
 
-bool RemoteLogger::sendData(const char* buffer, size_t bufferSize)
-{
-  size_t pos = 0;
-  while(pos < bufferSize) {
-    ssize_t written = write(d_socket, buffer + pos, bufferSize - pos);
-    if (written == -1) {
-      int res = errno;
-      if (res == EWOULDBLOCK || res == EAGAIN) {
-        return false;
-      }
-      else if (res != EINTR) {
-        reconnect();
-        return false;
-      }
-    }
-    else if (written == 0) {
-      reconnect();
-      return false;
-    }
-    else {
-      pos += (size_t) written;
-    }
-  }
-
-  return true;
-}
-
 void RemoteLogger::worker()
 {
   if (d_asyncConnect) {
